@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: iso2022.c,v 1.30 2003/04/29 14:53:38 nalin Exp $"
+#ident "$Id: iso2022.c,v 1.33 2003/05/06 05:17:56 nalin Exp $"
 #include "../config.h"
 #include <sys/types.h>
 #include <errno.h>
@@ -255,8 +255,7 @@ static const struct _vte_iso2022_map _vte_iso2022_map_U[] = {
 
 /* Japanese.  JIS X 0201-1976 ("Roman" set), per RFC 1468/2237. */
 static const struct _vte_iso2022_map _vte_iso2022_map_J[] = {
-	{'\\', 0x203e},
-	{'~',  GDK_yen},
+#include "unitable.JIS0201"
 };
 /* Japanese.  JIS X 0208-1978, per RFC 1468/2237. */
 static const struct _vte_iso2022_map _vte_iso2022_map_wide_at[] = {
@@ -529,6 +528,7 @@ _vte_iso2022_map_get(gunichar mapname,
 		}
 		width = 2; /* CJKV expects 2 bytes -> 2 columns */
 		bytes = 2;
+		*and_mask = 0xf7f7f;
 		break;
 	case 'A' + WIDE_FUDGE:
 		if (map == NULL) {
@@ -537,6 +537,7 @@ _vte_iso2022_map_get(gunichar mapname,
 		}
 		width = 2; /* CJKV expects 2 bytes -> 2 columns */
 		bytes = 2;
+		*and_mask = 0xf7f7f;
 		break;
 	case 'B' + WIDE_FUDGE:
 		if (map == NULL) {
@@ -545,6 +546,7 @@ _vte_iso2022_map_get(gunichar mapname,
 		}
 		width = 2; /* CJKV expects 2 bytes -> 2 columns */
 		bytes = 2;
+		*and_mask = 0xf7f7f;
 		break;
 	case 'C' + WIDE_FUDGE:
 		if (map == NULL) {
@@ -553,6 +555,7 @@ _vte_iso2022_map_get(gunichar mapname,
 		}
 		width = 2; /* CJKV expects 2 bytes -> 2 columns */
 		bytes = 2;
+		*and_mask = 0xf7f7f;
 		break;
 	case 'D' + WIDE_FUDGE:
 		if (map == NULL) {
@@ -561,6 +564,7 @@ _vte_iso2022_map_get(gunichar mapname,
 		}
 		width = 2; /* CJKV expects 2 bytes -> 2 columns */
 		bytes = 2;
+		*and_mask = 0xf7f7f;
 		break;
 	case 'G' + WIDE_FUDGE:
 		if (map == NULL) {
@@ -570,6 +574,7 @@ _vte_iso2022_map_get(gunichar mapname,
 		/* Return the plane number as part of the "or" mask. */
 		g_assert(or_mask != NULL);
 		*or_mask = 0x10000;
+		*and_mask = 0xf7f7f;
 		width = 2; /* CJKV expects 2 bytes -> 2 columns */
 		bytes = 2;
 		break;
@@ -581,6 +586,7 @@ _vte_iso2022_map_get(gunichar mapname,
 		/* Return the plane number as part of the "or" mask. */
 		g_assert(or_mask != NULL);
 		*or_mask = 0x20000;
+		*and_mask = 0xf7f7f;
 		width = 2; /* CJKV expects 2 bytes -> 2 columns */
 		bytes = 2;
 		break;
@@ -592,6 +598,7 @@ _vte_iso2022_map_get(gunichar mapname,
 		/* Return the plane number as part of the "or" mask. */
 		g_assert(or_mask != NULL);
 		*or_mask = 0x30000;
+		*and_mask = 0xf7f7f;
 		width = 2; /* CJKV expects 2 bytes -> 2 columns */
 		bytes = 2;
 		break;
@@ -603,6 +610,7 @@ _vte_iso2022_map_get(gunichar mapname,
 		/* Return the plane number as part of the "or" mask. */
 		g_assert(or_mask != NULL);
 		*or_mask = 0x40000;
+		*and_mask = 0xf7f7f;
 		width = 2; /* CJKV expects 2 bytes -> 2 columns */
 		bytes = 2;
 		break;
@@ -614,6 +622,7 @@ _vte_iso2022_map_get(gunichar mapname,
 		/* Return the plane number as part of the "or" mask. */
 		g_assert(or_mask != NULL);
 		*or_mask = 0x50000;
+		*and_mask = 0xf7f7f;
 		width = 2; /* CJKV expects 2 bytes -> 2 columns */
 		bytes = 2;
 		break;
@@ -625,6 +634,7 @@ _vte_iso2022_map_get(gunichar mapname,
 		/* Return the plane number as part of the "or" mask. */
 		g_assert(or_mask != NULL);
 		*or_mask = 0x60000;
+		*and_mask = 0xf7f7f;
 		width = 2; /* CJKV expects 2 bytes -> 2 columns */
 		bytes = 2;
 		break;
@@ -636,6 +646,7 @@ _vte_iso2022_map_get(gunichar mapname,
 		/* Return the plane number as part of the "or" mask. */
 		g_assert(or_mask != NULL);
 		*or_mask = 0x70000;
+		*and_mask = 0xf7f7f;
 		width = 2; /* CJKV expects 2 bytes -> 2 columns */
 		bytes = 2;
 		break;
@@ -733,7 +744,9 @@ _vte_iso2022_state_set_codeset(struct _vte_iso2022_state *state,
 			  codeset, state->target_codeset);
 		return;
 	}
-	g_iconv_close(state->conv);
+	if (state->conv != (GIConv) -1) {
+		g_iconv_close(state->conv);
+	}
 	state->codeset = g_quark_to_string(g_quark_from_string(codeset));
 	state->conv = conv;
 }
@@ -747,8 +760,10 @@ _vte_iso2022_state_get_codeset(struct _vte_iso2022_state *state)
 void
 _vte_iso2022_state_free(struct _vte_iso2022_state *state)
 {
-	g_iconv_close(state->conv);
-	state->conv = NULL;
+	if (state->conv != ((GIConv) -1)) {
+		g_iconv_close(state->conv);
+	}
+	state->conv = (GIConv) -1;
 	state->native_codeset = state->utf8_codeset = state->codeset = NULL;
 	state->target_codeset = NULL;
 	state->g[3] = 'B';
@@ -784,6 +799,8 @@ _vte_iso2022_find_nextctl(const char *p, size_t length)
 	ret = _vte_iso2022_better(ret, memchr(p, '\r', length));
 	ret = _vte_iso2022_better(ret, memchr(p, '\016', length));
 	ret = _vte_iso2022_better(ret, memchr(p, '\017', length));
+	ret = _vte_iso2022_better(ret, memchr(p, 0x8e, length));
+	ret = _vte_iso2022_better(ret, memchr(p, 0x8f, length));
 	return ret;
 }
 
@@ -821,7 +838,7 @@ _vte_iso2022_fragment_input(struct _vte_buffer *input, GArray *blocks)
 		p = nextctl;
 		valids = NULL;
 		sequence_length = -1;
-		switch (nextctl[0]) {
+		switch ((guint8)nextctl[0]) {
 		case '\n':
 		case '\r':
 		case '\016':
@@ -832,6 +849,12 @@ _vte_iso2022_fragment_input(struct _vte_buffer *input, GArray *blocks)
 			/* SI */
 			sequence_length = 1;
 			break;
+		case 0x8e:
+		case 0x8f:
+			/* SS2 - 8bit */
+			/* SS3 - 8bit */
+			sequence_length = 1;
+			break;
 		case '\033':
 			if (q - nextctl < 2) {
 				/* Inconclusive. */
@@ -1092,6 +1115,9 @@ process_cdata(struct _vte_iso2022_state *state, guchar *cdata, gsize length,
 		/* Append the unichars to the array. */
 		for (i = 0; &buf[i] < (char*)outbuf; i += sizeof(gunichar)) {
 			c = *(gunichar*)(buf + i);
+			if (c == '\0') {
+				continue;
+			}
 			if (_vte_iso2022_is_ambiguous(c)) {
 				width = ambiguous_width;
 				c = _vte_iso2022_set_encoded_width(c, width);
@@ -1229,6 +1255,24 @@ process_control(struct _vte_iso2022_state *state, guchar *ctl, gsize length,
 			if (_vte_debug_on(VTE_DEBUG_SUBSTITUTION)) {
 				fprintf(stderr, "\tSI\n");
 			}
+#endif
+			break;
+		case 0x8e:
+			/* SS2 - 8bit */
+			state->override = 2;
+#ifdef VTE_DEBUG
+			if (_vte_debug_on(VTE_DEBUG_SUBSTITUTION)) {
+				fprintf(stderr, "\tSS2 (8-bit)\n");
+			}
+#endif
+			break;
+		case 0x8f:
+			/* SS3 - 8bit */
+			state->override = 3;
+#ifdef VTE_DEBUG
+			if (_vte_debug_on(VTE_DEBUG_SUBSTITUTION)) {
+				fprintf(stderr, "\tSS3 (8-bit)\n");
+			}
 #endif
 			break;
 		default:
@@ -1462,10 +1506,20 @@ _vte_iso2022_process(struct _vte_iso2022_state *state,
 		case _vte_iso2022_cdata:
 #ifdef VTE_DEBUG
 			if (_vte_debug_on(VTE_DEBUG_SUBSTITUTION)) {
-				fprintf(stderr, "%3ld %3ld CDATA \"%.*s\".\n",
+				int j;
+				fprintf(stderr, "%3ld %3ld CDATA \"%.*s\"",
 					block.start, block.end,
 					(int) (block.end - block.start),
 					input->bytes + block.start);
+				fprintf(stderr, " (");
+				for (j = block.start; j < block.end; j++) {
+					if (j > block.start) {
+						fprintf(stderr, ", ");
+					}
+					fprintf(stderr, "0x%02x",
+						input->bytes[j]);
+				}
+				fprintf(stderr, ")\n");
 			}
 #endif
 			initial = 0;
