@@ -16,12 +16,14 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: matcher.c,v 1.2 2003/04/30 18:35:37 nalin Exp $"
+#ident "$Id: matcher.c,v 1.3 2003/05/05 23:23:46 nalin Exp $"
 
+#include "../config.h"
 #include <sys/types.h>
 #include <stdio.h>
 #include <string.h>
 #include <glib-object.h>
+#include "debug.h"
 #include "matcher.h"
 #include "table.h"
 #include "trie.h"
