@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: pty.c,v 1.42 2003/04/24 02:50:54 nalin Exp $"
+#ident "$Id: pty.c,v 1.43 2003/05/05 02:51:51 nalin Exp $"
 #include "../config.h"
 #include <sys/types.h>
 #include <sys/ioctl.h>
@@ -218,6 +218,9 @@ n_read(int fd, void *buffer, size_t count)
 			switch (errno) {
 			case EINTR:
 			case EAGAIN:
+#ifdef ERESTART
+			case ERESTART:
+#endif
 				break;
 			default:
 				return -1;
@@ -247,6 +250,9 @@ n_write(int fd, const void *buffer, size_t count)
 			switch (errno) {
 			case EINTR:
 			case EAGAIN:
+#ifdef ERESTART
+			case ERESTART:
+#endif
 				break;
 			default:
 				return -1;
