@@ -18,9 +18,10 @@
 
 /* The interfaces in this file are subject to change at any time. */
 
-#ident "$Id: vtedraw.c,v 1.6 2003/04/14 23:31:59 nalin Exp $"
+#ident "$Id: vtedraw.c,v 1.11 2003/05/06 07:38:39 nalin Exp $"
 
 #include "../config.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <glib.h>
@@ -37,13 +38,6 @@
 struct _vte_draw_impl
 *_vte_draw_impls[] = {
 	&_vte_draw_skel,
-#if GTK_CHECK_VERSION(2,2,0)
-#ifndef X_DISPLAY_MISSING
-#ifdef HAVE_GL
-	/* &_vte_draw_gl, */
-#endif
-#endif
-#endif
 #ifndef X_DISPLAY_MISSING
 #ifdef HAVE_XFT2
 	&_vte_draw_xft,
@@ -56,6 +50,13 @@ struct _vte_draw_impl
 	&_vte_draw_pango_x,
 #endif
 #endif
+#if GTK_CHECK_VERSION(2,2,0)
+#ifndef X_DISPLAY_MISSING
+#ifdef HAVE_GL
+	&_vte_draw_gl,
+#endif
+#endif
+#endif
 };
 
 struct _vte_draw *
@@ -112,7 +113,7 @@ _vte_draw_free(struct _vte_draw *draw)
 		draw->widget = NULL;
 	}
 	draw->started = FALSE;
-	
+
 	g_free(draw);
 }
 
@@ -127,9 +128,19 @@ _vte_draw_get_visual(struct _vte_draw *draw)
 GdkColormap *
 _vte_draw_get_colormap(struct _vte_draw *draw)
 {
+	GdkColormap *colormap;
 	g_return_val_if_fail(draw->impl != NULL, NULL);
 	g_return_val_if_fail(draw->impl->get_colormap != NULL, NULL);
-	return draw->impl->get_colormap(draw);
+	colormap = draw->impl->get_colormap(draw);
+	if (colormap) {
+		return colormap;
+	}
+#if GTK_CHECK_VERSION(2,2,0)
+	colormap = gdk_screen_get_default_colormap(gdk_screen_get_default());
+#else
+	colormap = gdk_colormap_get_system();
+#endif
+	return colormap;
 }
 
 void
@@ -163,11 +174,17 @@ _vte_draw_set_background_color(struct _vte_draw *draw, GdkColor *color)
 }
 
 void
-_vte_draw_set_background_pixbuf(struct _vte_draw *draw, GdkPixbuf *pixbuf)
+_vte_draw_set_background_image(struct _vte_draw *draw,
+			       enum VteBgSourceType type,
+			       GdkPixbuf *pixbuf,
+			       const char *filename,
+			       const GdkColor *color,
+			       double saturation)
 {
 	g_return_if_fail(draw->impl != NULL);
-	g_return_if_fail(draw->impl->set_background_pixbuf != NULL);
-	draw->impl->set_background_pixbuf(draw, pixbuf);
+	g_return_if_fail(draw->impl->set_background_image != NULL);
+	draw->impl->set_background_image(draw, type, pixbuf, filename,
+					 color, saturation);
 }
 void
 _vte_draw_clear(struct _vte_draw *draw, gint x, gint y, gint width, gint height)
@@ -256,5 +273,5 @@ _vte_draw_set_scroll(struct _vte_draw *draw, gint x, gint y)
 {
 	g_return_if_fail(draw->impl != NULL);
 	g_return_if_fail(draw->impl->set_scroll != NULL);
-	return draw->impl->set_scroll(draw, x, y);
+	draw->impl->set_scroll(draw, x, y);
 }
