@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: vtefc.c,v 1.7 2003/04/28 18:58:55 nalin Exp $"
+#ident "$Id: vtefc.c,v 1.8 2003/05/05 23:23:46 nalin Exp $"
 
 #include "../config.h"
 
@@ -325,7 +325,10 @@ _vte_fc_defaults_from_rdb(FcPattern *pattern)
  * and append them to the array. */
 gboolean
 _vte_fc_patterns_from_pango_font_desc(const PangoFontDescription *font_desc,
-				      GArray *pattern_array)
+				      GArray *pattern_array,
+				      _vte_fc_defaults_cb defaults_cb,
+				      gpointer defaults_data)
+
 {
 	FcPattern *pattern, *match, *tmp;
 	FcFontSet *fontset;
@@ -353,6 +356,11 @@ _vte_fc_patterns_from_pango_font_desc(const PangoFontDescription *font_desc,
 	/* Add any defaults which are hard-coded in fontconfig. */
 	FcDefaultSubstitute(pattern);
 
+	/* Add any defaults via a callback. */
+	if (defaults_cb != NULL) {
+		defaults_cb(pattern, defaults_data);
+	}
+
 	/* Get a sorted list of patterns, duplicate them, and append them
 	 * to the passed-in array. */
 	fontset = FcFontSort(NULL, pattern, FcTrue, NULL, &result);
